@@ -1119,4 +1119,226 @@ def set_powers(grid_obj,bus_name,s_cplx,mode='urisi_3ph'):
         q = s_cplx.imag
         for ph in ['a','b','c']:
             grid_obj.set_value(f'p_{bus_name}_{ph}',p/3)
-            grid_obj.set_value(f'q_{bus_name}_{ph}',q/3)        
\ No newline at end of file
+            grid_obj.set_value(f'q_{bus_name}_{ph}',q/3)  
+            
+def update_loads(grid,data_input):
+    
+    if type(data_input) == dict:
+        data = data_input
+        
+    if type(data_input) == str:
+        data_input = open(data_input).read().replace("'",'"')
+        data = json.loads(data_input)
+        
+    if 'loads' in data:
+        for load in data['loads']:
+            
+            if load['type'] == "1P+N":
+                bus = load['bus']
+                kVA = load['kVA']
+                pf = load['pf']
+                p = kVA*1000*pf
+                q = np.sqrt((kVA*1000)**2 - p**2)*np.sign(pf)
+
+                grid.set_value(f'p_load_{bus}_1',p)
+                grid.set_value(f'q_load_{bus}_1',q)
+
+def report_v(grid,data_input,show=True,model='urisi'):
+    '''
+    
+
+    Parameters
+    ----------
+    grid : pydae object
+        Pydae object modelling a grid.
+    data_input : if dict, a dictionary with the grid parameters
+                 if string, the path to the .json file containing grid parameters
+
+    show : string, optional
+        If report is print or not.
+        
+    model : string, optional
+        Type of implemented model. The default is 'urisi'.
+
+    Returns
+    -------
+    dict with the results.
+
+    '''
+    
+    
+    if type(data_input) == dict:
+        data = data_input
+        
+    if type(data_input) == str:
+        data_input = open(data_input).read().replace("'",'"')
+        data = json.loads(data_input)
+        
+    buses_dict = {}
+        
+    for bus in data['buses']:
+        if f"v_{bus['bus']}_n_r" in grid.y_ini_list:
+            v_n_r,v_n_i = grid.get_mvalue([f"v_{bus['bus']}_n_r",f"v_{bus['bus']}_n_i"])
+            v_n_g = v_n_r + 1j*v_n_i
+        else:
+            v_n_g = 0.0
+        #for ph in ['a','b','c']:
+        
+        ph = 'a'
+        v_a_r,v_a_i =  grid.get_mvalue([f"v_{bus['bus']}_{ph}_r",f"v_{bus['bus']}_{ph}_i"])
+        v_a_g = v_a_r + 1j*v_a_i
+        v_a_n = v_a_g - v_n_g
+        v_a_m = np.abs(v_a_n)
+        v_a_a = np.rad2deg(np.angle(v_a_n))
+
+        ph = 'b'
+        v_b_r,v_b_i =  grid.get_mvalue([f"v_{bus['bus']}_{ph}_r",f"v_{bus['bus']}_{ph}_i"])
+        v_b_g = v_b_r + 1j*v_b_i
+        v_b_n = v_b_g - v_n_g
+        v_b_m = np.abs(v_b_n)
+        v_b_a = np.rad2deg(np.angle(v_b_n))
+
+        ph = 'c'
+        v_c_r,v_c_i =  grid.get_mvalue([f"v_{bus['bus']}_{ph}_r",f"v_{bus['bus']}_{ph}_i"])
+        v_c_g = v_c_r + 1j*v_c_i
+        v_c_n = v_c_g - v_n_g
+        v_c_m = np.abs(v_c_n)
+        v_c_a = np.rad2deg(np.angle(v_c_n))
+ 
+        alpha = alpha = np.exp(2.0/3*np.pi*1j)
+        v_0 =  1/3*(v_a_g+v_b_g+v_c_g)
+        v_1 = 1.0/3.0*(v_a_g + v_b_g*alpha + v_c_g*alpha**2)
+        v_2 = 1.0/3.0*(v_a_g + v_b_g*alpha**2 + v_c_g*alpha)
+        
+        # compute unbalanced as in Kersting 1ers edition pg. 266
+        v_m_array = [v_a_m,v_b_m,v_c_m]
+        v_m_min = np.min(v_m_array)
+        v_m_max = np.max(v_m_array)
+        max_dev = v_m_max - v_m_min
+        v_avg = np.sum(v_m_array)/3
+        unbalance = max_dev/v_avg
+        
+        bus[f"v_{bus['bus']}_{'a'}n"] = v_a_m
+        
+        if show:
+            print(f"v_{bus['bus']}_{'a'}n: {v_a_m:7.1f}| {v_a_a:6.1f}º V,    v_{bus['bus']}_{'a'}g: {np.abs(v_a_g):7.1f}| {np.angle(v_a_g,deg=True):6.1f}º V,    v_1 = {np.abs(v_1):7.1f} V, unb = {unbalance*100:3.2f}%")
+            print(f"v_{bus['bus']}_{'b'}n: {v_b_m:7.1f}| {v_b_a:6.1f}º V,    V_{bus['bus']}_{'b'}g: {np.abs(v_b_g):7.1f}| {np.angle(v_b_g,deg=True):6.1f}º V,    v_2 = {np.abs(v_2):7.1f} V")
+            print(f"v_{bus['bus']}_{'c'}n: {v_c_m:7.1f}| {v_c_a:6.1f}º V,    V_{bus['bus']}_{'c'}g: {np.abs(v_c_g):7.1f}| {np.angle(v_c_g,deg=True):6.1f}º V,    v_0 = {np.abs(v_0):7.1f} V")                   
+            
+            print(f"  V_{bus['bus']}_ng: {np.abs(v_n_g):8.1f}| {np.angle(v_n_g,deg=True):8.1f}º V")
+        
+        buses_dict[bus['bus']] = {f"v_{bus['bus']}_{'a'}n":v_b_m,f"v_{bus['bus']}_{'b'}n":v_b_m,f"v_{bus['bus']}_{'c'}n":v_c_m}
+        buses_dict[bus['bus']].update({f'v_unb':unbalance,'v_ng':np.abs(v_n_g)})
+
+    return buses_dict        
+        
+def report_trafos(grid,data_input,model='urisi'):
+    '''
+    
+
+    Parameters
+    ----------
+    grid : pydae object
+        Pydae object modelling a grid.
+    data_input : if dict, a dictionary with the grid parameters
+                 if string, the path to the .json file containing grid parameters
+
+    model : string, optional
+        Type of implemented model. The default is 'urisi'.
+
+    Returns
+    -------
+    None.
+
+    '''
+    
+    
+    if type(data_input) == dict:
+        data = data_input
+        
+    if type(data_input) == str:
+        data_input = open(data_input).read().replace("'",'"')
+        data = json.loads(data_input)
+        
+    for trafo in data['transformers']:
+        
+        bus_j_name = trafo['bus_j']
+        bus_k_name = trafo['bus_k']
+        
+        for ph in ['a','b','c']:
+            i_r,i_i = grid.get_mvalue([f"i_t_{bus_j_name}_{bus_k_name}_1_{ph}_r",
+                                       f"i_t_{bus_j_name}_{bus_k_name}_1_{ph}_i"])
+            i_1 = i_r + 1j*i_i
+            i_m = np.abs(i_1)
+            phi = np.arctan2(i_i,i_r)
+            print(f"I_1_{ph}: {i_m:8.1f} |{np.rad2deg(phi):6.1f}º A")
+            
+        for ph in ['a','b','c']:
+            i_r,i_i = grid.get_mvalue([f"i_t_{bus_j_name}_{bus_k_name}_2_{ph}_r",
+                                       f"i_t_{bus_j_name}_{bus_k_name}_2_{ph}_i"])
+            i_1 = i_r + 1j*i_i
+            i_m = np.abs(i_1)
+            phi = np.arctan2(i_i,i_r)
+            print(f"I_2_{ph}: {i_m:8.1f} |{np.rad2deg(phi):6.1f}º A")
+            
+        if f"i_t_{bus_j_name}_{bus_k_name}_2_n_r" in grid.outputs_list:
+            i_r,i_i = grid.get_mvalue([f"i_t_{bus_j_name}_{bus_k_name}_2_n_r",
+                                       f"i_t_{bus_j_name}_{bus_k_name}_2_n_i"])
+            i_1 = i_r + 1j*i_i
+            i_m = np.abs(i_1)
+            phi = np.arctan2(i_i,i_r)            
+            print(f"I_2_n: {i_m:8.1f} |{np.rad2deg(phi):6.1f}º A")
+
+
+def load_shape(grid,data_input,model='urisi'):
+    '''
+    
+
+    Parameters
+    ----------
+    grid : pydae object
+        Pydae object modelling a grid.
+    data_input : if dict, a dictionary with the grid parameters
+                 if string, the path to the .json file containing grid parameters
+
+    model : string, optional
+        Type of implemented model. The default is 'urisi'.
+
+    Returns
+    -------
+    None.
+
+    '''
+    
+    
+    if type(data_input) == dict:
+        data = data_input
+        
+    if type(data_input) == str:
+        data_input = open(data_input).read().replace("'",'"')
+        data = json.loads(data_input)
+        
+def read_shapes(data_input):
+    '''
+    
+
+    Parameters
+    ----------
+    data_input : if string, the path to the .json file containing grid parameters
+
+    Returns
+    -------
+    dict with the shapes.
+
+    '''
+    
+    data_input = open(data_input).read().replace("'",'"')
+    data = json.loads(data_input)
+    shapes = {}
+    for item in data['shapes']:
+        values = np.array(data['shapes'][item])
+        shapes.update({item:{'t':values[:,0],'val':values[:,1]}})
+        
+    
+    
+    return shapes
\ No newline at end of file
