@@ -7,6 +7,8 @@ sin = np.sin
 cos = np.cos
 atan2 = np.arctan2
 sqrt = np.sqrt 
+sign = np.sign 
+exp = np.exp
 
 
 class {name}_class: 
@@ -50,9 +52,27 @@ class {name}_class:
         self.u_run_list = self.inputs_run_list
         self.u_run_values_list = self.inputs_run_values_list
         self.N_u = len(self.u_run_list)
+        Fx_ini_rows,Fx_ini_cols,Fy_ini_rows,Fy_ini_cols,Gx_ini_rows,Gx_ini_cols,Gy_ini_rows,Gy_ini_cols = nonzeros()
+
+        self.Fx_ini_rows = np.array(Fx_ini_rows) 
+        if len(Fx_ini_rows) == 1: 
+            self.Fx_ini_rows = np.array([[Fx_ini_rows]]).reshape(1,) 
+            self.Fx_ini_cols = np.array([[Fx_ini_cols]]).reshape(1,)  
+            
+        self.Fx_ini_cols = np.array(Fx_ini_cols)
+        self.Fy_ini_rows = np.array(Fy_ini_rows)        
+        self.Fy_ini_cols = np.array(Fy_ini_cols)
+        self.Gx_ini_rows = np.array(Gx_ini_rows)        
+        self.Gx_ini_cols = np.array(Gx_ini_cols)
+        self.Gy_ini_rows = np.array(Gy_ini_rows)        
+        self.Gy_ini_cols = np.array(Gy_ini_cols)
+        
+        
+        self.yini2urun = list(set(self.inputs_run_list).intersection(set(self.y_ini_list)))
+        self.uini2yrun = list(set(self.y_run_list).intersection(set(self.inputs_ini_list)))
+
         self.update() 
-
-
+                
     def update(self): 
 
         self.N_steps = int(np.ceil(self.t_end/self.Dt)) 
@@ -103,8 +123,22 @@ class {name}_class:
               ('Z', np.float64, (self.N_store+1,self.N_z)),
               ('iters', np.float64, (self.N_store+1,1)),
               ('store', np.int64),
+              ('Fx_ini_rows', np.int64, self.Fx_ini_rows.shape),
+              ('Fx_ini_cols', np.int64, self.Fx_ini_cols.shape),
+              ('Fy_ini_rows', np.int64, self.Fy_ini_rows.shape),
+              ('Fy_ini_cols', np.int64, self.Fy_ini_cols.shape),
+              ('Gx_ini_rows', np.int64, self.Gx_ini_rows.shape),
+              ('Gx_ini_cols', np.int64, self.Gx_ini_cols.shape),
+              ('Gy_ini_rows', np.int64, self.Gy_ini_rows.shape),
+              ('Gy_ini_cols', np.int64, self.Gy_ini_cols.shape),
+              ('Ac_ini', np.float64, ((self.N_x+self.N_y,self.N_x+self.N_y))),   
+              ('fg', np.float64, ((self.N_x+self.N_y,1))),  
              ]
 
+
+
+        
+        
         values = [
                 self.t_end,                          
                 self.Dt,
@@ -152,6 +186,16 @@ class {name}_class:
                 np.zeros((self.N_store+1,self.N_z)),   # Z
                 np.zeros((self.N_store+1,1)),          # iters
                 1,
+                self.Fx_ini_rows,       
+                self.Fx_ini_cols,
+                self.Fy_ini_rows,       
+                self.Fy_ini_cols,
+                self.Gx_ini_rows,        
+                self.Gx_ini_cols,
+                self.Gy_ini_rows,       
+                self.Gy_ini_cols,
+                np.zeros((self.N_x+self.N_y,self.N_x+self.N_y)),  
+                np.zeros((self.N_x+self.N_y,1)),
                 ]  
 
         dt += [(item,np.float64) for item in self.params_list]
@@ -170,6 +214,8 @@ class {name}_class:
         xy0 = np.zeros((self.N_x+self.N_y,))
         self.ini_dae_jacobian_nn(xy0)
         self.run_dae_jacobian_nn(xy0)
+        
+
 
     def load_params(self,data_input):
 
@@ -184,15 +230,42 @@ class {name}_class:
         self.data = data
         for item in self.data:
             self.struct[0][item] = self.data[item]
-            self.params_values_list[self.params_list.index(item)] = self.data[item]
+            if item in self.params_list:
+                self.params_values_list[self.params_list.index(item)] = self.data[item]
+            elif item in self.inputs_ini_list:
+                self.inputs_ini_values_list[self.inputs_ini_list.index(item)] = self.data[item]
+            elif item in self.inputs_run_list:
+                self.inputs_run_values_list[self.inputs_run_list.index(item)] = self.data[item]
+            else: 
+                print(f'parameter or input {item} not found')
 
+    def save_params(self,file_name = 'parameters.json'):
+        params_dict = {}
+        for item in self.params_list:
+            params_dict.update({item:self.get_value(item)})
 
+        params_dict_str = json.dumps(params_dict, indent=4)
+        with open(file_name,'w') as fobj:
+            fobj.write(params_dict_str)
+
+    def save_inputs_ini(self,file_name = 'inputs_ini.json'):
+        inputs_ini_dict = {}
+        for item in self.inputs_ini_list:
+            inputs_ini_dict.update({item:self.get_value(item)})
+
+        inputs_ini_dict_str = json.dumps(inputs_ini_dict, indent=4)
+        with open(file_name,'w') as fobj:
+            fobj.write(inputs_ini_dict_str)
 
     def ini_problem(self,x):
         self.struct[0].x[:,0] = x[0:self.N_x]
         self.struct[0].y_ini[:,0] = x[self.N_x:(self.N_x+self.N_y)]
-        ini(self.struct,2)
-        ini(self.struct,3)       
+        if self.compile:
+            ini(self.struct,2)
+            ini(self.struct,3)       
+        else:
+            ini.py_func(self.struct,2)
+            ini.py_func(self.struct,3)                   
         fg = np.vstack((self.struct[0].f,self.struct[0].g))[:,0]
         return fg
 
@@ -200,12 +273,21 @@ class {name}_class:
         t = self.struct[0].t
         self.struct[0].x[:,0] = x[0:self.N_x]
         self.struct[0].y_run[:,0] = x[self.N_x:(self.N_x+self.N_y)]
-        run(t,self.struct,2)
-        run(t,self.struct,3)
-        run(t,self.struct,10)
-        run(t,self.struct,11)
-        run(t,self.struct,12)
-        run(t,self.struct,13)
+        
+        if self.compile:
+            run(t,self.struct,2)
+            run(t,self.struct,3)
+            run(t,self.struct,10)
+            run(t,self.struct,11)
+            run(t,self.struct,12)
+            run(t,self.struct,13)
+        else:
+            run.py_func(t,self.struct,2)
+            run.py_func(t,self.struct,3)
+            run.py_func(t,self.struct,10)
+            run.py_func(t,self.struct,11)
+            run.py_func(t,self.struct,12)
+            run.py_func(t,self.struct,13)            
         
         fg = np.vstack((self.struct[0].f,self.struct[0].g))[:,0]
         return fg
@@ -244,8 +326,12 @@ class {name}_class:
     def ini_dae_jacobian(self,x):
         self.struct[0].x[:,0] = x[0:self.N_x]
         self.struct[0].y_ini[:,0] = x[self.N_x:(self.N_x+self.N_y)]
-        ini(self.struct,10)
-        ini(self.struct,11)       
+        if self.compile:
+            ini(self.struct,10)
+            ini(self.struct,11) 
+        else:
+            ini.py_func(self.struct,10)
+            ini.py_func(self.struct,11)             
         A_c = np.block([[self.struct[0].Fx_ini,self.struct[0].Fy_ini],
                         [self.struct[0].Gx_ini,self.struct[0].Gy_ini]])
         return A_c
@@ -393,7 +479,7 @@ class {name}_class:
                 self.xy_prev[self.y_ini_list.index(item)+self.N_x] = xy_0_dict[item]
                 
             
-    def initialize(self,events=[{}],xy0=0):
+    def initialize(self,events=[{}],xy0=0,compile=True):
         '''
         
 
@@ -419,6 +505,9 @@ class {name}_class:
             DESCRIPTION.
 
         '''
+        
+        self.compile = compile
+        
         # simulation parameters
         self.struct[0].it = 0       # set time step to zero
         self.struct[0].it_store = 0 # set storage to zero
@@ -497,16 +586,28 @@ class {name}_class:
             else:
                 sol = sopt.root(self.run_problem, xy0, method=self.sopt_root_method)
 
-            # evaluate f and g
-            run(0.0,self.struct,2)
-            run(0.0,self.struct,3)                
-
-            
-            # evaluate run jacobians 
-            run(0.0,self.struct,10)
-            run(0.0,self.struct,11)                
-            run(0.0,self.struct,12) 
-            run(0.0,self.struct,14) 
+            if self.compile:
+                # evaluate f and g
+                run(0.0,self.struct,2)
+                run(0.0,self.struct,3)                
+    
+                # evaluate run jacobians 
+                run(0.0,self.struct,10)
+                run(0.0,self.struct,11)                
+                run(0.0,self.struct,12) 
+                run(0.0,self.struct,14) 
+                
+            else:
+                # evaluate f and g
+                run.py_func(0.0,self.struct,2)
+                run.py_func(0.0,self.struct,3)                
+    
+                # evaluate run jacobians 
+                run.py_func(0.0,self.struct,10)
+                run.py_func(0.0,self.struct,11)                
+                run.py_func(0.0,self.struct,12) 
+                run.py_func(0.0,self.struct,14)                 
+                
              
             # post process result    
             T = self.struct[0]['T'][:self.struct[0].it_store]
@@ -570,31 +671,62 @@ class {name}_class:
                         
         return mvalue
     
-    def set_value(self,name,value):
-        if name in self.inputs_run_list:
-            self.struct[0][name] = value
-        if name in self.params_list:
-            self.struct[0][name] = value
+    def set_value(self,name_,value):
+        if name_ in self.inputs_run_list:
+            self.struct[0][name_] = value
+            return
+        elif name_ in self.params_list:
+            self.struct[0][name_] = value
+            return
+        elif name_ in self.inputs_ini_list:
+            self.struct[0][name_] = value
+            return 
+        else:
+            print(f'Input or parameter {name_} not found.')
+
+    def set_values(self,dictionary):
+        
+        for item in dictionary:
+            self.set_value(item,dictionary[item])
             
-    def report_x(self,value_format='5.2f'):
+            
+    def report_x(self,value_format='5.2f', decimals=2):
         for item in self.x_list:
-            print(f'{item:5s} = {self.get_value(item):5.2f}')
+            print(f'{item:5s} = {self.get_value(item):5.{decimals}f}')
 
-    def report_y(self,value_format='5.2f'):
+    def report_y(self,value_format='5.2f', decimals=2):
         for item in self.y_run_list:
-            print(f'{item:5s} = {self.get_value(item):5.2f}')
+            print(f'{item:5s} = {self.get_value(item):5.{decimals}f}')
             
-    def report_u(self,value_format='5.2f'):
+    def report_u(self,value_format='5.2f', decimals=2):
         for item in self.inputs_run_list:
-            print(f'{item:5s} = {self.get_value(item):5.2f}')
+            print(f'{item:5s} = {self.get_value(item):5.{decimals}f}')
 
-    def report_z(self,value_format='5.2f'):
+    def report_z(self,value_format='5.2f', decimals=2):
         for item in self.outputs_list:
-            print(f'{item:5s} = {self.get_value(item):5.2f}')
+            print(f'{item:5s} = {self.get_value(item):5.{decimals}f}')
 
-    def report_params(self,value_format='5.2f'):
+    def report_params(self,value_format='5.2f', decimals=2):
         for item in self.params_list:
-            print(f'{item:5s} = {self.get_value(item):5.2f}')
+            print(f'{item:5s} = {self.get_value(item):5.{decimals}f}')
             
     def get_x(self):
-        return self.struct[0].x
\ No newline at end of file
+        return self.struct[0].x
+    
+    def ss(self):
+        
+        ssate(self.struct,self.xy_prev.reshape(len(self.xy_prev),1))
+        
+        ## y_ini to y_run
+        self.struct[0].y_run = self.struct[0].y_ini
+        
+        ## y_ini to u_run
+        for item in self.yini2urun:
+            self.struct[0][item] = self.struct[0].y_ini[self.y_ini_list.index(item)]
+                
+        ## u_ini to y_run
+        for item in self.uini2yrun:
+            self.struct[0].y_run[self.y_run_list.index(item)] = self.struct[0][item]
+
+
+
